#!/bin/bash
#
# Copyright (C) 2013-2018 NetFlow Logic. All rights reserved.
#
# NetFlow Optimizer Daemon install/uninstall script
#
#==============================================================
# Options:
# -i - install NetFlow Optimizer daemon
# -u - uninstall NetFlow Optimizer daemon
#==============================================================

declare -i e_INCORRECT_ARG=70
declare -i e_UNSUPPORTED_ID=71
declare -i e_ALREADY_INSTALLED=72
declare -i e_NOT_INSTALLED=73
declare -i e_CANNOT_DETECT_DISTRIBUTOR_ID=74
declare -i e_SYSTEMD_DIR_FOR_SERVICES_NOT_FOUND=75

ARG0="${0}"
while [ -h "${ARG0}" ]; do
	ls=$(ls -ld "${ARG0}")
	link=$(expr "${ls}" : '.*-> \(.*\)$')
	if $(expr "${link}" : '/.*' > /dev/null); then
		ARG0="${link}"
	else
		ARG0="$(dirname ${ARG0})/${link}"
	fi
done

function isSystemd
{
	local -i __errCode=1

	whereis systemctl | grep "^systemctl:[[:space:]]\+.*systemctl.*$" > /dev/null
	__errCode=$?

	return ${__errCode}
}

function isOpenRC
{
	local -i __errCode=1

	whereis rc-service | grep "^rc\-service:[[:space:]]\+.*rc\-service.*$" > /dev/null
	__errCode=$?

	return ${__errCode}
}

declare -i openRCFound=1
declare -i systemdFound=1

declare appRoot="$(readlink -f $(dirname ${ARG0}))"

# Mode: install or uninstall
declare mode=""
declare msgNFIRunning="NetFlow Optimizer server is running!\nPlease, stop it before NetFlow Optimizer daemon ${mode}."
declare msgAlreadyInstalled="NetFlow Optimizer daemon is already installed."
declare msgNotInstalled="NetFlow Optimizer daemon was not installed."
declare msgCannotDetectDistributorId="Cannot detect Distributor ID."
declare msgSystemdForServicesDirNotFound="Systemd directory for services not found."
declare msgUseInitSystem=""
declare msgOperation=""
declare msgOperationSuccess=""
declare msgOperationFail=""
declare msgUnsupportedDistributorId=""
declare -i systemdFound=1

isSystemd
systemdFound=$?

isOpenRC
openRCFound=$?

for arg in "${*}"
do
	case ${arg} in
		-i)
			mode="install"
			msgOperation="Installing NetFlow Optimizer daemon..."
			msgOperationSuccess="NetFlow Optimizer daemon has been successfully installed."
			msgOperationFail="NetFlow Optimizer daemon install FAILED! Error code -- "
			msgUnsupportedDistributorId="NetFlow Optimizer daemon install is not supported for Distributor ID '"
			if (( ${systemdFound} == 0 ));then
				msgUseInitSystem="Systemd found. Using it for registering NetFlow Optimizer daemon."
			elif (( ${openRCFound} == 0 ));then
				msgUseInitSystem="OpenRC found. Using it for registering NetFlow Optimizer daemon."
			else
				msgUseInitSystem="Systemd not found. Using UNIX System V for registering NetFlow Optimizer daemon."
			fi
		;;
		-u)
			mode="uninstall"
			msgOperation="Uninstalling NetFlow Optimizer daemon..."
			msgOperationSuccess="NetFlow Optimizer daemon has been successfully uninstalled."
			msgOperationFail="NetFlow Optimizer daemon uninstall FAILED! Error code -- "
			msgUnsupportedDistributorId="NetFlow Optimizer daemon uninstall is not supported for Distributor ID '"
			if (( ${systemdFound} == 0 ));then
				msgUseInitSystem="Systemd found. Using it for unregistering NetFlow Optimizer daemon."
			elif (( ${openRCFound} == 0 ));then
				msgUseInitSystem="OpenRC found. Using it for unregistering NetFlow Optimizer daemon."
			else
				msgUseInitSystem="Systemd not found. Using UNIX System V for unregistering NetFlow Optimizer daemon."
			fi
		;;
		*)
			echo "Script usage:"
			echo "${0} -i | -u"
			echo "-i - install NetFlow Optimizer daemon"
			echo "-u - uninstall NetFlow Optimizer daemon"
			exit ${e_INCORRECT_ARG}
		;;
	esac
done


declare jreHome="${appRoot}/java/jre8"
declare jreHomeEnv=""
declare jreTARName=""

function checkLastOperationResult 
{
	local __res=$?
	if [[ ${__res} != 0 ]] ; then
		echo -e '\E[31;40m'"${msgOperationFail}${__res}."
		tput sgr0
		exit ${__res}
	fi
}

function uninstallTomcat7
{
	case ${id} in
		rhel | suse)
			__cnt=$(/sbin/chkconfig --list tomcat7_nfi 2>&1 | grep -c "^tomcat7_nfi[[:space:]]\+0:\(off\|on\)[[:space:]]\+1:\(off\|on\)[[:space:]]\+2:\(off\|on\)[[:space:]]\+3:\(off\|on\)[[:space:]]\+4:\(off\|on\)[[:space:]]\+5:\(off\|on\)[[:space:]]\+6:\(off\|on\)")
			if [[ ${__cnt} -eq 1 ]]; then
				/sbin/chkconfig --del tomcat7_nfi 2>&1 > /dev/null
				checkLastOperationResult
			fi
		;;
		debian)
			__cnt2=$(ls /etc/rc2.d/S* | grep -c "S*tomcat7_nfi$")
			__cnt3=$(ls /etc/rc3.d/S* | grep -c "S*tomcat7_nfi$")
			__cnt4=$(ls /etc/rc4.d/S* | grep -c "S*tomcat7_nfi$")
			__cnt5=$(ls /etc/rc5.d/S* | grep -c "S*tomcat7_nfi$")
			((__cnt = __cnt2 + __cnt3 + __cnt4 + __cnt5))

			if [[ ${__cnt} -gt 0 ]]; then
				update-rc.d -f tomcat7_nfi remove 2>&1 > /dev/null
				checkLastOperationResult
			fi
		;;
	esac
	if [[ -h "/etc/init.d/tomcat7_nfi" ]]; then
		rm /etc/init.d/tomcat7_nfi
	fi
}

# main part

pushd ${appRoot} >> /dev/null

echo "${msgUseInitSystem}"
echo "${msgOperation}"

if (( ${systemdFound} == 0 ));then
	case ${mode} in
		install)
			if [[ ! -d "/etc/systemd/system" ]]; then
				echo "${msgSystemdForServicesDirNotFound}"
				exit ${e_SYSTEMD_DIR_FOR_SERVICES_NOT_FOUND}
			elif [[ ! -f "/etc/systemd/system/tomcat_nfo.service" ]]; then
				declare vTmp="${appRoot//\//\\/}"
				vTmp="${vTmp//\./\\.}"
				cat ${appRoot}/tomcat_nfo.service.template | sed "s/\[nfo_app_root\]/"${vTmp}"/" > /etc/systemd/system/tomcat_nfo.service
				checkLastOperationResult
				systemctl daemon-reload 2>&1 > /dev/null
			else
				echo "${msgAlreadyInstalled}"
				exit ${e_ALREADY_INSTALLED}
			fi
		;;
		uninstall)
			if [[ -f "/etc/systemd/system/tomcat_nfo.service" ]]; then
				systemctl disable tomcat_nfo.service 2>&1 > /dev/null
				checkLastOperationResult
				rm /etc/systemd/system/tomcat_nfo.service 2>&1 > /dev/null
				checkLastOperationResult
				systemctl daemon-reload 2>&1 > /dev/null
				systemctl reset-failed 2>&1 > /dev/null
			else
				echo "${msgNotInstalled}"
				exit ${e_NOT_INSTALLED}
			fi
		;;
	esac
else
	# Distributor ID:
	# rhel -- RHEL, CentOS or Oracle Linux
	# debian -- Debian or Ubuntu
	# suse -- SUSE Linux Enterprise Server
	# gentoo -- Gentoo Linux
	declare id=""
	is_lsb_release=$(whereis lsb_release | grep -c "lsb_release:.\+")
	if (( is_lsb_release == 0 ));then
		is_os_release=$(find /etc/ -maxdepth 1 -mindepth 1 -type f -name 'os-release' | grep -c "\/etc\/os-release")
		if (( is_os_release == 0 ));then
			is_etc_release=$(find /etc/ -maxdepth 1 -mindepth 1 -type f -name '*-release' | grep -c "\/etc\/.*-release")
			if (( is_etc_release == 0 ));then
				if [[ -f "/etc/debian_version" ]]; then
					strId="debian"
				else
					echo "${msgCannotDetectDistributorId}"
					exit ${e_CANNOT_DETECT_DISTRIBUTOR_ID}
				fi
			else
				sId=$(sed -n '1p' /etc/*-release)
				strId=$(echo "${sId}" | tr [[:upper:]] [[:lower:]] | sed "s/[[:space:]]//g")
			fi
		else
			sId=$(sed -n '/^ID\=/p;' /etc/os-release | sed 's/ID\=\"\?\([[:alnum:]]\+\)\"\?$/\1/')
			strId="${sId}"
		fi
	else
		sId=$(lsb_release -i | sed "s/Distributor[[:space:]]*ID:[[:space:]]*//")
		strId=$(echo "${sId}" | tr [[:upper:]] [[:lower:]])
	fi

	case "${strId}" in
		redhat*) id="rhel";;
		rhel*) id="rhel";;
		centos*) id="rhel";;
		ol) id="rhel";;
		oracle*) id="rhel";;
		debian*) id="debian";;
		ubuntu*) id="debian";;
		suse*) id="suse";;
		opensuse*) id="suse";;
		sles*) id="suse";;
		gentoo*) id="gentoo";;
		*)
			echo "${msgUnsupportedDistributorId}${sId}"
			exit ${e_UNSUPPORTED_ID}
		;;
	esac

	declare -i cnt=0

	case ${id} in
		rhel)
			cnt=$(/sbin/chkconfig --list tomcat_nfo 2>&1 | grep -c "^tomcat_nfo[[:space:]]\+0:\(off\|on\)[[:space:]]\+1:\(off\|on\)[[:space:]]\+2:\(off\|on\)[[:space:]]\+3:\(off\|on\)[[:space:]]\+4:\(off\|on\)[[:space:]]\+5:\(off\|on\)[[:space:]]\+6:\(off\|on\)")

			case ${mode} in
				install)
					uninstallTomcat7
					if [[ ! -h "/etc/init.d/tomcat_nfo" ]]; then
						ln -s ${appRoot}/nfi_daemon_rhel.sh /etc/init.d/tomcat_nfo 2>&1 > /dev/null
						checkLastOperationResult
					else
						if [[ ${cnt} -eq 1 ]]; then
							echo "${msgAlreadyInstalled}"
							exit ${e_ALREADY_INSTALLED}
						fi
					fi

					if [[ ${cnt} -eq 1 ]]; then
						/sbin/chkconfig --del tomcat_nfo 2>&1 > /dev/null
						checkLastOperationResult
					fi
					/sbin/chkconfig --add tomcat_nfo 2>&1 > /dev/null
					checkLastOperationResult
				;;
				uninstall)
					if [[ -h "/etc/init.d/tomcat_nfo" ]]; then
						if [[ ${cnt} -eq 1 ]]; then
							/sbin/chkconfig --del tomcat_nfo 2>&1 > /dev/null
							checkLastOperationResult
						fi
						rm /etc/init.d/tomcat_nfo
					else
						echo "${msgNotInstalled}"
						exit ${e_NOT_INSTALLED}
					fi
				;;
			esac
		;;
		debian)
			cnt2=$(ls /etc/rc2.d/S* | grep -c "S*tomcat_nfo$")
			cnt3=$(ls /etc/rc3.d/S* | grep -c "S*tomcat_nfo$")
			cnt4=$(ls /etc/rc4.d/S* | grep -c "S*tomcat_nfo$")
			cnt5=$(ls /etc/rc5.d/S* | grep -c "S*tomcat_nfo$")
			((cnt = cnt2 + cnt3 + cnt4 + cnt5))

			case ${mode} in
				install)
					uninstallTomcat7
					if [[ ! -h "/etc/init.d/tomcat_nfo" ]]; then
						ln -s ${appRoot}/nfi_daemon_deb.sh /etc/init.d/tomcat_nfo 2>&1 > /dev/null
						checkLastOperationResult
					else
						if [[ ${cnt} -ne 0 ]]; then
							echo "${msgAlreadyInstalled}"
							exit ${e_ALREADY_INSTALLED}
						fi
					fi

					if [[ ${cnt} -gt 0 ]]; then
						update-rc.d -f tomcat_nfo remove 2>&1 > /dev/null
						checkLastOperationResult
					fi
					update-rc.d tomcat_nfo start 85 2 3 4 5 . stop 01 0 1 6 . 2>&1 > /dev/null
					checkLastOperationResult
				;;
				uninstall)
					if [[ -h "/etc/init.d/tomcat_nfo" ]]; then
						if [[ ${cnt} -gt 0 ]]; then
							update-rc.d -f tomcat_nfo remove 2>&1 > /dev/null
							checkLastOperationResult
						fi
						rm /etc/init.d/tomcat_nfo
					else
						echo "${msgNotInstalled}"
						exit ${e_NOT_INSTALLED}
					fi
				;;
			esac
		;;
		suse)
			cnt=$(/sbin/chkconfig --list tomcat_nfo 2>&1 | grep -c "^tomcat_nfo[[:space:]]\+0:\(off\|on\)[[:space:]]\+1:\(off\|on\)[[:space:]]\+2:\(off\|on\)[[:space:]]\+3:\(off\|on\)[[:space:]]\+4:\(off\|on\)[[:space:]]\+5:\(off\|on\)[[:space:]]\+6:\(off\|on\)")

			case ${mode} in
				install)
					uninstallTomcat7
					if [[ ! -h "/etc/init.d/tomcat_nfo" ]]; then
						ln -s ${appRoot}/nfi_daemon_suse.sh /etc/init.d/tomcat_nfo 2>&1 > /dev/null
						checkLastOperationResult
					else
						if [[ ${cnt} -eq 1 ]]; then
							echo "${msgAlreadyInstalled}"
							exit ${e_ALREADY_INSTALLED}
						fi
					fi

					if [[ ${cnt} -eq 1 ]]; then
						/sbin/chkconfig --del tomcat_nfo 2>&1 > /dev/null
						checkLastOperationResult
					fi
					/sbin/chkconfig --add tomcat_nfo 2>&1 > /dev/null
					checkLastOperationResult
				;;
				uninstall)
					if [[ -h "/etc/init.d/tomcat_nfo" ]]; then
						if [[ ${cnt} -eq 1 ]]; then
							/sbin/chkconfig --del tomcat_nfo 2>&1 > /dev/null
							checkLastOperationResult
						fi
						rm /etc/init.d/tomcat_nfo 2>&1 > /dev/null
					else
						echo "${msgNotInstalled}"
						exit ${e_NOT_INSTALLED}
					fi
				;;
			esac
		;;
		gentoo)
			cnt=$(rc-service -r tomcat_nfo 2>&1 | grep -c "^/etc/init.d/tomcat_nfo$")

			case ${mode} in
				install)
					if [[ ! -h "/etc/init.d/tomcat_nfo" ]]; then
						ln -s ${appRoot}/nfi_daemon_gentoo.sh /etc/init.d/tomcat_nfo 2>&1 > /dev/null
						checkLastOperationResult
					else
						if [[ ${cnt} -eq 1 ]]; then
							echo "${msgAlreadyInstalled}"
							exit ${e_ALREADY_INSTALLED}
						fi
					fi

					if [[ ${cnt} -eq 1 ]]; then
						rc-update del tomcat_nfo 2>&1 > /dev/null
						checkLastOperationResult
					fi
					rc-update add tomcat_nfo default 2>&1 > /dev/null
					checkLastOperationResult
				;;
				uninstall)
					if [[ -h "/etc/init.d/tomcat_nfo" ]]; then
						if [[ ${cnt} -eq 1 ]]; then
							rc-update del tomcat_nfo 2>&1 > /dev/null
							checkLastOperationResult
						fi
						rm /etc/init.d/tomcat_nfo
					else
						echo "${msgNotInstalled}"
						exit ${e_NOT_INSTALLED}
					fi
				;;
			esac
		;;
	esac
fi

popd > /dev/null

echo "${msgOperationSuccess}"
