#!/bin/bash
#
# Copyright (C) 2012 - 2018 NetFlow Logic. All rights reserved.
#
# NetFlow Optimizer server startup/shutdown script
# description: NetFlow Optimizer server daemon

cd $(dirname "${0}")
declare NFC_DAEMON=nfc_svr
declare NFC_CONFIG=../etc/server.cfg
# Directory for the logs
declare dirLogs="../../logs/"
# Directory for the rotated logs
declare dirBackup="${dirLogs}bak/"
# The number of the logs to be stored
declare -i numToStore=10
# Max log size
# 200m
declare -i maxSize=209715200
# Polling interval for the monitoring NFO server log file size
declare -i pollingInterval=1
# Lock file name for NFO server log file size monitor
declare monitorLockFile="../etc/.monitor.loc"
# Log file name mask
declare logNameMask="server.????-??-??.??_??_??.??.log"

# Get current date
getDate()
{
	local ___dt=$(date "+%Y-%m-%d.%H_%M_%S.%N")
	echo ${___dt:0:22}
}

# Rotate log
# in/out $1: rotated log name/a new log name
rotateLog()
{
	local -i ___numToDel=0
	local ___logNameOut="${1}"
	local ___logName="${!___logNameOut}"
	local ___logNameOld="${___logName}"
	local ___f=""
	local -i ___removeOnly=${2}

	cp -ft "${dirBackup}" "${___logName}"
	if [[ ${___removeOnly} -eq 0 ]];then
		cat /dev/null > "${___logName}"
		___logName="${dirLogs}server.$(getDate).log"
		mv -fT "${___logNameOld}" "${___logName}"
	fi

	___numToDel=$(find ${dirBackup} -maxdepth 1 -mindepth 1 -type f -name "${logNameMask}" | wc -l)-${numToStore}+1
	if [[ ${___numToDel} -gt 0 ]];then
		for ___f in $(find ${dirBackup} -maxdepth 1 -mindepth 1 -type f -name "${logNameMask}" | sort | head -n ${___numToDel})
		do
			if [[ -f "${___f}" ]];then
				rm -f ${___f}
			fi
		done
	fi

	eval ${___logNameOut}='${___logName}'

	return 0
}

# Get daemon pid
# in $1: daemon
# out $2: pid
getPid()
{
	local ___daemon=${1}
	local ___pid=${2}
	eval ${___pid}=$(ps -eo pid,comm | awk "/$___daemon\$/  {print \$1}")
}

# Check if it is own daemon
# in $1: pid
# in $2: daemon
# out $3: isOwn (0 - own daemon, 1 - foreign daemon)
isOwnDaemon()
{
	local ___pid=${1}
    local ___daemon=${2}
	local ___isOwn=${3}
	if [[ "$(ps -p $___pid -o args=)" == "$(pwd)/$___daemon"* ]]; then
		eval ${___isOwn}=0
	else
		eval ${___isOwn}=1
	fi
}

# Stop daemon
# in $1: daemon
# return 0 - success, 1 - fail
stopDaemon()
{
	local __daemon=${1}
	local __res=0
	local __pid=""
	local __isOwn=1
	local __sleep=0

	getPid ${__daemon} __pid
	if [[ "${__pid}" != "" ]]; then
		isOwnDaemon ${__pid} ${__daemon} __isOwn
		if [[ ${__isOwn} == 0 ]]; then
			kill -15 $__pid
			__res=1

			__sleep=0
			while [ ${__sleep} -lt 30 -a ${__res} -eq 1 ]; do
				# wait 30 seconds until daemon is terminated
				getPid ${__daemon} __pid
				if [[ "${__pid}" == "" ]]; then
					__res=0
				else
					echo "Waiting for processes to stop...";
					sleep 5
					__sleep=$(expr ${__sleep} + 5)
				fi
			done

			if [ "${__res}" != 0 ]; then
				# daemon hasn't been terminated, kill it
				kill -9 ${__pid}
				getPid ${__daemon} __pid
				if [[ "${__pid}" != "" ]]; then
					echo "NetFlow Optimizer server can't be stopped (PID: ${__pid})." 1>&2
				else
					__res=0
				fi
			fi
		else
			echo "NetFlow Optimizer server can't be stopped as it was launched from another location." 1>&2
		fi
	else
		echo "NetFlow Optimizer is not running."
	fi
	return ${__res}
}

# Start NetFlow Optimizer server
# return 0 - success, 1 - if NetFlow Optimizer server is already running or previous countroller can't be stopped.
startServer() 
{
	local __pid=""
	local __f=""
	local __isOwn=1
	local __logName=""
	local __logNameOld=""

	getPid ${NFC_DAEMON} __pid
	if [[ "${__pid}" == "" ]]; then
		export LD_LIBRARY_PATH=".:$LD_LIBRARY_PATH"

		echo "Starting NetFlow Optimizer server..."

# Remove the lock for NFO server log file size monitor to stop it, if it is still running
		if [[ -f "${monitorLockFile}" ]];then
			# allow deleting lock file
			chmod 700 "${monitorLockFile}"
			rm -f "${monitorLockFile}"
			# give to the previous instance of monitor a time for finalizing
			sleep ${pollingInterval}
		fi

# Rotate log files from previous starts of NFO server, if they exist
		local __i=0
		for __f in $(find ${dirLogs} -maxdepth 1 -mindepth 1 -type f -name "${logNameMask}" | sort)
		do
			__logNameOld="${__f}"
			rotateLog __f ${__i}
			if [[ -f "${__logNameOld}" ]];then
				rm -f ${__logNameOld}
			fi
			if [[ ${__i} -eq 0 ]];then
				__logName="${__f}"
			fi
			((__i++))
		done

		if [[ -z "${__logName}" ]];then
			__logName="${dirLogs}server.$(getDate).log"
		fi

# Run NFO in subshell: stdout and stderr redirected to a log file
		(
			echo "$(date '+%F %T') Starting server..."
			while read -r __p __l ; do echo "${__l}" ; break ; done < <(whereis -b ionice) | grep -c ".\+"
			local -i __retVal=$?
			if [[ ${__retVal} -eq 0 ]];then
				ionice -c 0 -n 0 $(pwd)/${NFC_DAEMON} -cfg ${NFC_CONFIG}
			else
				$(pwd)/${NFC_DAEMON} -cfg ${NFC_CONFIG}
			fi
			echo "$(date '+%F %T') Server stopped."
		) >> ${__logName} 2>&1 &
		__pid=$!

# Run monitoring NFO server log file size in subshell. Rotate, if max size exceeded
# Subshell output is redirected to /dev/null to let to close stdout of main script
		(
			# create lock file
			touch "${monitorLockFile}"
			# disallow deleting lock file
			chmod 400 "${monitorLockFile}"
			local -i __retVal
			ps -p ${__pid} >> /dev/null
			__retVal=$?
			# monitor, till NFO server process and lock file exist
			while [[ ${__retVal} -eq 0 && -f "${monitorLockFile}" ]]
			do
				if [[ $(stat -c%s "${__logName}") -gt ${maxSize} ]]; then
					rotateLog __logName 0
				fi
				sleep ${pollingInterval}
				ps -p ${__pid} >> /dev/null
				__retVal=$?
			done

			if [[ -f "${monitorLockFile}" ]];then
				# allow deleting lock file
				chmod 700 "${monitorLockFile}"
				rm -f "${monitorLockFile}"
			fi
		) > /dev/null 2>&1 &

		echo "NetFlow Optimizer server started"
		return 0
	else
		isOwnDaemon ${__pid} ${NFC_DAEMON} __isOwn
		if [[ ${__isOwn} == 0 ]]; then
			echo "NetFlow Optimizer server is already running." >&2
		else
			echo "Another instance of NetFlow Optimizer server is running." >&2
		fi
		return 1
	fi
}

# Stop NetFlow Optimizer server
# return 0 - success, 1 - fail
stopServer() 
{
	stopDaemon ${NFC_DAEMON}
	return $?
}

# Get NetFlow Optimizer server status
status() 
{
	local __pid=""
	local __isOwn=1
	getPid ${NFC_DAEMON} __pid
	if [[ "${__pid}" != "" ]]; then
		isOwnDaemon ${__pid} ${NFC_DAEMON} __isOwn
		if [[ ${__isOwn} == 0 ]]; then
			echo "NetFlow Optimizer is running (PID: ${__pid})."
		else
			echo "Another instance of NetFlow Optimizer server is running."
		fi
	else
		echo "NetFlow Optimizer is not running."
	fi
	return 0
}

### main logic ###
case "${1}" in
	start)
		startServer
		;;
	stop)
		stopServer
		;;
	status)
		status
		;;
	restart)
		stopServer
		startServer
    	;;
	*)
		echo $"Usage: ${0} {start|stop|restart|status}"
		exit 1
esac

exit $?
