#!/bin/bash
#
# Copyright (C) 2012 - 2019 NetFlow Logic. All rights reserved.
#
# NetFlow Optimizer setup script
#
#==============================================================
# Options:
# -i - install NetFlow Optimizer
# -u - uninstall NetFlow Optimizer
#==============================================================

declare -i e_INCORRECT_ARG=70
declare -i e_ALREADY_INSTALLED=72
declare -i e_NOT_INSTALLED=73
declare -i e_NFI_STARTED=75

ARG0="${0}"
while [ -h "${ARG0}" ]; do
	ls=$(ls -ld "${ARG0}")
	link=$(expr "${ls}" : '.*-> \(.*\)$')
	if $(expr "${link}" : '/.*' > /dev/null); then
		ARG0="${link}"
	else
		ARG0="$(dirname ${ARG0})/${link}"
	fi
done

declare -i openRCFound=1
declare -i systemdFound=1

declare appRoot="$(readlink -f $(dirname ${ARG0}))"

declare mode=""
declare jreTARName="zulu8.38.0.13-ca-jre8.0.212-linux_x64.tar.gz"

declare msgConfiguring=""
declare msgConfiguringSuccess=""
declare msgConfiguringFail=""

declare msgInfo="The NetFlow Optimizer web interface is at "
declare msgNFIStart="Starting NetFlow Optimizer..."
declare msgNFIRunning="NetFlow Optimizer server is running!\nPlease, stop it before setup."
declare msgDaemonRunAdvice="Your Linux OS doesn't support systemd. \
To install/uninstall NetFlow Optimizer service on RHEL, CentOS, Oracle Linux, \
SUSE, Debian or Ubuntu, please use commands '${appRoot}/register_daemon.sh -i' or \
'${appRoot}/register_daemon.sh -u' respectively."
declare msgSystemdDaemonRunAdvice="Your Linux OS supports systemd. \
To install/uninstall NetFlow Optimizer service, please use commands '${appRoot}/register_daemon.sh -i' or \
'${appRoot}/register_daemon.sh -u' respectively."
declare msgOpenRCDaemonRunAdvice="Your Linux OS supports OpenRC. \
To install/uninstall NetFlow Optimizer service, please use commands '${appRoot}/register_daemon.sh -i' or \
'${appRoot}/register_daemon.sh -u' respectively."
declare msgNFIRunAdvice="To get all available commands for NetFlow Optimizer service, please \
use command '${appRoot}/nfi.sh' or '/etc/init.d/tomcat_nfo'."
declare msgSystemdNFIRunAdvice="To manage NetFlow Optimizer service, please \
use command 'systemctl start|stop|restart|status tomcat_nfo.service'. \
For details, type 'systemctl --help'."
declare msgOpenRCNFIRunAdvice="To manage NetFlow Optimizer service, please \
use command 'rc-service tomcat_nfo start|stop|restart|status'. \
For details, type 'rc-service --help'."
declare msgAlreadyInstalled="NetFlow Optimizer is already installed."

function checkLastOperationResult
{
	local __res=$?
	if [[ ${__res} != 0 ]] ; then
		echo -e '\E[31;40m'"${msgConfiguringFail}${__res}."
		tput sgr0
		exit ${__res}
	fi
}

function isSystemd
{
	local -i __errCode=1

	whereis systemctl | grep "^systemctl:[[:space:]]\+.*systemctl.*$" > /dev/null
	__errCode=$?

	return ${__errCode}
}

function isOpenRC
{
	local -i __errCode=1

	whereis rc-service | grep "^rc\-service:[[:space:]]\+.*rc\-service.*$" > /dev/null
	__errCode=$?

	return ${__errCode}
}

# main part
for arg in "${*}"
do
	case ${arg} in
		-i)
			mode="install"
			msgConfiguring="Installing NetFlow Optimizer..."
			msgConfiguringSuccess="NetFlow Optimizer has been successfully installed."
			msgConfiguringFail="NetFlow Optimizer install FAILED! Error code -- "
		;;
		-u)
			mode="uninstall"
			msgConfiguring="Uninstalling NetFlow Optimizer..."
			msgConfiguringSuccess="NetFlow Optimizer has been successfully uninstalled."
			msgConfiguringFail="NetFlow Optimizer uninstall FAILED! Error code -- "
		;;
		*)
			echo "Script usage:"
			echo "${0} -i | -u"
			echo "-i - install NetFlow Optimizer"
			echo "-u - uninstall NetFlow Optimizer"
			exit ${e_INCORRECT_ARG}
		;;
	esac
done

pushd ${appRoot} >> /dev/null

echo "${msgConfiguring}"

is_nfi_running=$(./server/bin/flowintegrator.sh status | grep -c '^NetFlow .* is running.*$')
if [[ ${is_nfi_running} -gt 0 ]]; then
	echo -e '\E[31;40m'"${msgNFIRunning}"
	tput sgr0
	exit ${e_NFI_STARTED}
fi

isSystemd
systemdFound=$?

if (( ${systemdFound} == 0 ));then
	if [[ -f "/etc/systemd/system/tomcat_nfo.service" ]]; then
		systemctl is-active tomcat_nfo.service | grep -c '^active$' 2>&1 > /dev/null
		is_jsvc_running=$?
		if (( ${is_jsvc_running} == 0 )); then
			systemctl stop tomcat_nfo.service
		fi
	elif [[ -f "./nfi.sh" ]]; then
		./nfi.sh status 2>&1 >&3 3>&- | grep -c '^NetFlow .* daemon is running.*$' 2>&1 > /dev/null
		is_jsvc_running=$?
		if (( ${is_jsvc_running} == 0 )) ; then
			./nfi.sh stop 2>&1 | > /dev/null
			checkLastOperationResult
		fi
	fi
else
	isOpenRC
	openRCFound=$?

	if [[ -h "/etc/init.d/tomcat7_nfi" ]]; then
		/etc/init.d/tomcat7_nfi status | grep -c '^NetFlow .* daemon is running.*$' 2>&1 > /dev/null
		is_jsvc_running=$?
		if (( ${is_jsvc_running} == 0 )) ; then
			/etc/init.d/tomcat7_nfi stop
			checkLastOperationResult
		fi
	fi
	if [[ -h "/etc/init.d/tomcat_nfo" ]]; then
		/etc/init.d/tomcat_nfo status | grep -c '^NetFlow .* daemon is running.*$' 2>&1 > /dev/null
		is_jsvc_running=$?
		if (( ${is_jsvc_running} == 0 )) ; then
			/etc/init.d/tomcat_nfo stop
			checkLastOperationResult
		fi
	elif [[ -f "./nfi.sh" ]]; then
		./nfi.sh status 2>&1 >&3 3>&- | grep -c '^NetFlow .* daemon is running.*$' > /dev/null
		is_jsvc_running=$?
		if (( ${is_jsvc_running} == 0 )) ; then
			./nfi.sh stop 2>&1 | > /dev/null
			checkLastOperationResult
		fi
	fi
fi

case ${mode} in
	install)

		if [[ -d "java/jre7/" ]];then
			rm -rf "java/jre7/" >> /dev/null
		fi
		if [[ -d "java/jre8/" ]];then
			rm -rf "java/jre8/" >> /dev/null
		fi

		tar zxvf java/${jreTARName} -C java --no-same-owner >> /dev/null
		checkLastOperationResult
		jdkDir=$(tar tzf java/${jreTARName} | head -1 | sed -e 's/\/.*//')
		checkLastOperationResult
		mv java/${jdkDir}/ java/jre8/ >> /dev/null
		checkLastOperationResult

		if [[ -d "tomcat/webapps/NetFlowIntegrator/" ]]; then
			rm -rf "tomcat/webapps/NetFlowIntegrator/" >> /dev/null
			checkLastOperationResult
		fi

		if [[ -f "tomcat/webapps/NetFlowIntegrator.war" ]]; then
			rm -f "tomcat/webapps/NetFlowIntegrator.war" || :
		fi

		if [[ -d "tomcat/webapps/ROOT/" ]]; then
			rm -rf "tomcat/webapps/ROOT/" >> /dev/null
			checkLastOperationResult
		fi

		./register_daemon.sh -i
		retVal=$?
		if [[ ${retVal} -eq 0 ]]; then
			:
		elif [[ ${retVal} -eq ${e_ALREADY_INSTALLED} ]]; then
			./register_daemon.sh -u
			checkLastOperationResult
			./register_daemon.sh -i
			checkLastOperationResult
		else
			echo -e '\E[31;40m'"${msgConfiguringFail}${retVal}."
			tput sgr0
			exit ${retVal}
		fi

		echo "${msgConfiguringSuccess}"

		echo "${msgNFIStart}"
		if (( ${systemdFound} == 0 ));then
			systemctl start tomcat_nfo.service
			checkLastOperationResult
			systemctl enable tomcat_nfo.service 2>&1 > /dev/null
		else
			/etc/init.d/tomcat_nfo start
		fi
		checkLastOperationResult

		appPath="https://localhost:8443/"
		nodeName=$(uname -n)
		appPath=${appPath/"localhost"/${nodeName}}
		echo -e '\E[32;40m'"${msgInfo}${appPath}"
		if (( ${systemdFound} == 0 ));then
			echo "${msgSystemdDaemonRunAdvice}"
			echo "${msgSystemdNFIRunAdvice}"
		elif (( ${openRCFound} == 0 ));then
			echo "${msgOpenRCDaemonRunAdvice}"
			echo "${msgOpenRCNFIRunAdvice}"
		else
			echo "${msgDaemonRunAdvice}"
			echo "${msgNFIRunAdvice}"
		fi
		tput sgr0
	;;
	uninstall)
		./register_daemon.sh -u
		retVal=$?
		if [[ ${retVal} -eq 0 || ${retVal} -eq ${e_NOT_INSTALLED} ]]; then
			# Use bash instead sh (see code backup-conf.sh) to avoid the issues in DASH (Ubuntu, Debian)
			bash ./backup-conf.sh
		else
			echo -e '\E[31;40m'"${msgConfiguringFail}${retVal}."
			tput sgr0
			exit ${retVal}
		fi

		target=$(basename ${appRoot})
		if [[ "${target}" == "flowintegrator" ]]; then
			for d in $(find ${appRoot}/ -maxdepth 1 -type d -path "${appRoot}/*")
			do
				base=$(basename ${d})
				case ${base} in
					flowintegrator)
						find ${d}/ -maxdepth 1 -type f -name "*" -exec rm -f {} \;
					;;
					logs | etc | conf-backup)
						:
					;;
					server)
						for d1 in $(find ${d}/ -maxdepth 1 -type d -path "${d}/*")
						do
							base1=$(basename ${d1})
							case ${base1} in
								server)
									:
								;;
								converters | policies | validators)
									:
								;;
								rule_sets)
									find ${d1}/ -maxdepth 1 -type f \! -name "p_*.zip" -exec rm -f {} \;
								;;
								etc)
									fileSvrCfg="${d1}/server.cfg"
									if [[ -f "${fileSvrCfg}" ]]; then
										cp -f "${fileSvrCfg}" "${fileSvrCfg}.backup"
									fi
									find ${d1}/ -maxdepth 1 -type f \! -name "server.cfg.backup" -exec rm -f {} \;
								;;
                                mibs)
                                    find ${d1}/ -maxdepth 1 -type f -exec rm -f {} \;
                                ;;
								*)
									rm -rf "${d1}"
								;;
							esac
						done
					;;
					tomcat)
						for d1 in $(find ${d}/ -maxdepth 1 -type d -path "${d}/*")
						do
							base1=$(basename ${d1})
							case ${base1} in
								tomcat)
									find ${d1}/ -maxdepth 1 -type f -name "*" -exec rm -f {} \;
								;;
								data)
									:
								;;
								*)
									rm -rf "${d1}"
								;;
							esac
						done
					;;
					*)
						rm -rf "${d}"
					;;
				esac
			done
		fi
		echo "${msgConfiguringSuccess}"
	;;
esac

popd > /dev/null
